/*
 * Created on 28. 7. 2017 9:44:08
 *
 */
package com.jh.boot.security.api;

import java.util.ArrayList;
import java.util.Base64;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.jh.boot.security.AuthError;
import com.jh.boot.security.AuthService;
import com.jh.boot.security.PasswordUtils;
import com.jh.boot.utils.Utils;
import com.jh.boot.web.error.BadRequestException;
import com.jh.boot.web.error.ErrorMessage;
import com.jh.boot.web.error.NotFoundException;
import com.jh.boot.web.error.RestApiException;

/**
 * The Class AuthController.
 *
 * @author Jan Hrabal
 */
@Controller
public class AuthApiController {
    
    /** The auth service. */
    @Autowired
    private AuthService authService;
    
    @Value("${auth.controller.signup.enabled:true}")
    private boolean signupEnabled;
    
    @Value("${auth.controller.reset.enabled:true}")
    private boolean resetEnabled;
    
	/**
	 * Do logout.
	 *
	 * @param session the session
	 */
	protected void doLogout(HttpSession session) {
		SecurityContextHolder.clearContext();
        if (session != null) {
            session.invalidate();
        }
	}
    
    
    /**
     * Logout api.
     *
     * @param session the session
     */
    @RequestMapping({"/auth/logout"})
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public void logoutApi(HttpSession session) {
        doLogout(session);
    }

    @RequestMapping(path = "/auth/login", method = RequestMethod.POST)
    public @ResponseBody LoginResponse login(@RequestBody LoginRequest login) {
    	Utils.sleep(250);
    	Authentication auth = null;
    	try {
    		auth = authService.authenticate(login.getUsername(), login.getPassword());
    	} catch (AuthenticationException e) {
    		//do nothing - will be resolved later
    	}
    	if (auth == null) {
    		throw new RestApiException(HttpStatus.FORBIDDEN, ErrorMessage.withCode("AUTH.BAD_USERNAME_OR_PASSWORD"));
    	}
    	return new LoginResponse("Authentication", "Basic " + Base64.getEncoder().encodeToString((login.getUsername() + ":" + login.getPassword()).getBytes()));
    }
    
    /**
     * Signup.
     *
     * @param signup the signup
     * @param locale the locale
     * @return the response entity
     */
    @RequestMapping(path = "/auth/signup", method = RequestMethod.POST)
    public @ResponseBody LoginResponse signup(@RequestBody Signup signup) {
    	if (!signupEnabled) {
    		throw new NotFoundException();
    	}
    	//validate
    	if (signup == null) {
    		throw new BadRequestException();
    	}
    	
    	List<AuthError> errors = new ArrayList<>();
    	
    	//TODO validate email
    	if (!StringUtils.hasText(signup.getLogin())) {
    		errors.add(new AuthError("username", "BadUsername"));
    	}
    	
    	if (!PasswordUtils.validatePassword(signup.getPassword())) {
    		errors.add(new AuthError("password", "BadPassword"));
    	}
    	
    	if (!errors.isEmpty()) {
//    		throw new BadRequestException(errors);
    		throw new BadRequestException();
    	}
    	
    	try {
    		authService.register(signup.getLogin(), signup.getPassword());
//			Authentication auth = authService.authenticate(signup.getLogin(), signup.getPassword());
//			SecurityContextHolder.getContext().setAuthentication(auth);

    	} catch (Exception e) {
    		e.printStackTrace();
    		errors.add(new AuthError(null, e.getMessage()));
    	}

    	return new LoginResponse("Authentication", "Basic " + Base64.getEncoder().encodeToString((signup.getLogin() + ":" + signup.getPassword()).getBytes()));
    }
    
    
    @RequestMapping(path = "/auth/resetRequest", method = RequestMethod.POST)
    public ResponseEntity<List<AuthError>> resetToken(@RequestBody ResetPassword request) {
    	if (!resetEnabled) {
    		return ResponseEntity.notFound().build();
    	}
    	Utils.sleep(250);
    	String token = authService.generateResetToken(request.getLogin());
    	return ResponseEntity.accepted().build();
    }
    
    
    @RequestMapping(path = "/auth/resetPassword", method = RequestMethod.POST)
    public ResponseEntity<List<AuthError>> reset(@RequestBody ResetPassword resetPassword) {
    	if (!resetEnabled) {
    		return ResponseEntity.notFound().build();
    	}
    	Utils.sleep(250);
    	String token = resetPassword.getToken();
    	if (!StringUtils.hasText(token)) {
    		return new ResponseEntity<>(Collections.singletonList(new AuthError(null, "NO_TOKEN")), HttpStatus.BAD_REQUEST);
    	}
    	
    	List<AuthError> errors = new ArrayList<>();
    	
    	try {
    		authService.resetPassword(resetPassword.getLogin(), resetPassword.getToken(), resetPassword.getPassword());
//    		if (errs == null || errs.isEmpty()) {
//    			Authentication auth = authService.authenticate(resetPassword.getLogin(), resetPassword.getPassword());
//    			SecurityContextHolder.getContext().setAuthentication(auth);
//    		} else {
//    			errors.addAll(errs);
//    		}
    	} catch (Exception e) {
    		e.printStackTrace();
    		errors.add(new AuthError(null, e.getMessage()));
    	}
    	
    	if (errors.isEmpty()) {
    		//return no errors
    		return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    	}
    	
		return new ResponseEntity<>(errors, HttpStatus.BAD_REQUEST);
    }
    
    
}
